@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, List
+from typing import TYPE_CHECKING
 
 from aasm.intermediate.graph import AdjRow, MatrixAgent, MatrixGraph
 
